@@ -100,10 +100,12 @@ fcitx_main_window_init(FcitxMainWindow* self)
 
     self->pagestore = _fcitx_main_window_create_model();
     self->pageview = gtk_icon_view_new_with_model(GTK_TREE_MODEL(self->pagestore));
-    
+
     gtk_icon_view_set_pixbuf_column(GTK_ICON_VIEW(self->pageview), PAGE_LIST_ICON);
     gtk_icon_view_set_text_column(GTK_ICON_VIEW(self->pageview), PAGE_LIST_NAME);
+#if GTK_CHECK_VERSION(2, 22, 0)
     gtk_icon_view_set_item_orientation(GTK_ICON_VIEW(self->pageview), GTK_ORIENTATION_VERTICAL);
+#endif
 
     _fcitx_main_window_add_im_page(self);
     _fcitx_main_window_add_config_file_page(self);
@@ -117,16 +119,13 @@ fcitx_main_window_init(FcitxMainWindow* self)
     gtk_misc_set_alignment(GTK_MISC(self->pagelabel), 0, 0.5);
 
     gtk_box_pack_start(GTK_BOX(self->vbox), self->pagelabel, FALSE, FALSE, 14);
-    GtkWidget* scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-    gtk_container_add(GTK_CONTAINER(scrolledwindow), self->pageview);
-    gtk_box_pack_start(GTK_BOX(hbox), scrolledwindow, FALSE, TRUE, 4);
+    gtk_box_pack_start(GTK_BOX(hbox), self->pageview, FALSE, TRUE, 4);
     gtk_box_pack_start(GTK_BOX(hbox), self->vbox, TRUE, TRUE, 8);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 8);
 
     gtk_container_add(GTK_CONTAINER(self), vbox);
 
-    gtk_icon_view_set_selection_mode(GTK_ICON_VIEW(self->pageview), GTK_SELECTION_SINGLE);
+    gtk_icon_view_set_selection_mode(GTK_ICON_VIEW(self->pageview), GTK_SELECTION_BROWSE);
     gtk_icon_view_set_item_padding(GTK_ICON_VIEW(self->pageview), 0);
     gtk_icon_view_set_margin(GTK_ICON_VIEW(self->pageview), 0);
     gtk_icon_view_set_column_spacing(GTK_ICON_VIEW(self->pageview), 0);
@@ -159,6 +158,8 @@ void fcitx_main_window_finalize(GObject* object)
 {
     FcitxMainWindow* self = FCITX_MAIN_WINDOW(object);
     utarray_free(self->addons);
+
+    G_OBJECT_CLASS (fcitx_main_window_parent_class)->finalize (object);
 }
 
 ConfigPage* _fcitx_main_window_add_page(FcitxMainWindow* self, const char* name, GtkWidget* widget, const char* stock)
@@ -188,7 +189,7 @@ void _fcitx_main_window_selection_changed_cb(GtkIconView* iconview, gpointer dat
     GtkTreeIter iter;
     ConfigPage* page;
     FcitxMainWindow* self = data;
-    
+
     GList* list = gtk_icon_view_get_selected_items(iconview);
 
     if (list) {
@@ -198,7 +199,7 @@ void _fcitx_main_window_selection_changed_cb(GtkIconView* iconview, gpointer dat
                            PAGE_LIST_NAME, &title,
                            PAGE_LIST_PAGE, &page,
                            -1);
-        
+
         gchar* text = g_strdup_printf("<b>%s</b>", title);
         gtk_label_set_markup(GTK_LABEL(self->pagelabel), text);
         g_free(text);
@@ -210,12 +211,8 @@ void _fcitx_main_window_selection_changed_cb(GtkIconView* iconview, gpointer dat
         gtk_widget_show_all(GTK_WIDGET(self));
 
         self->lastpage = page;
-    } else {
-        GtkTreePath* path = gtk_tree_model_get_path(GTK_TREE_MODEL(self->pagestore), &self->impage->iter);
-        gtk_icon_view_select_path(GTK_ICON_VIEW(self->pageview), path);
-        gtk_tree_path_free(path);
     }
-    
+
     g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
     g_list_free (list);
 }
@@ -443,33 +440,9 @@ void _fcitx_main_window_configure_button_clicked(GtkButton* button, gpointer dat
         gtk_tree_model_get(model, &iter,
                            LIST_ADDON, &addon,
                            -1);
-        gchar* config_desc_name = g_strdup_printf("%s.desc", addon->name);
-        FcitxConfigFileDesc* cfdesc = get_config_desc(config_desc_name);
-        g_free(config_desc_name);
-        gboolean configurable = (gboolean)(cfdesc != NULL || strlen(addon->subconfig) != 0);
-        if (configurable) {
-            GtkWidget* dialog = gtk_dialog_new_with_buttons(addon->generalname,
-                                GTK_WINDOW(self),
-                                GTK_DIALOG_MODAL,
-                                GTK_STOCK_OK,
-                                GTK_RESPONSE_OK,
-                                GTK_STOCK_CANCEL,
-                                GTK_RESPONSE_CANCEL,
-                                NULL
-                                                           );
-
-            gchar* config_file_name = g_strdup_printf("%s.config", addon->name);
-            FcitxConfigWidget* config_widget = fcitx_config_widget_new(cfdesc, "conf", config_file_name, addon->subconfig);
-            GtkWidget* content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-            gtk_box_pack_start(GTK_BOX(content_area), GTK_WIDGET(config_widget), TRUE, TRUE, 0);
-            g_free(config_file_name);
-            gtk_widget_set_size_request(GTK_WIDGET(config_widget), -1, 400);
-
-            g_signal_connect(dialog, "response",
-                             G_CALLBACK(fcitx_config_widget_response_cb),
-                             config_widget);
 
+        GtkWidget* dialog = fcitx_config_dialog_new(addon, GTK_WINDOW(self));
+        if (dialog)
             gtk_widget_show_all(GTK_WIDGET(dialog));
-        }
     }
 }
