@@ -34,6 +34,7 @@
 #include "config_widget.h"
 #include "keygrab.h"
 #include "sub_config_widget.h"
+#include "configdesc.h"
 
 #define _(s) gettext(s)
 #define D_(d, x) dgettext (d, x)
@@ -152,6 +153,7 @@ fcitx_config_widget_setup_ui(FcitxConfigWidget *self)
             gtk_grid_set_column_spacing(GTK_GRID(grid), 6);
             GtkWidget *plabel = gtk_label_new(D_(cfdesc->domain, cgdesc->groupName));
             GtkWidget *scrollwnd = gtk_scrolled_window_new(NULL, NULL);
+            g_object_set(G_OBJECT(scrollwnd), "shadow-type", GTK_SHADOW_NONE, NULL);
 
             gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwnd), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
             gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollwnd), grid);
@@ -174,19 +176,23 @@ fcitx_config_widget_setup_ui(FcitxConfigWidget *self)
 
                 switch (codesc->type) {
                 case T_Integer:
-                    inputWidget = gtk_spin_button_new_with_range(-1.0, 10000.0, 1.0);
+                    inputWidget = gtk_spin_button_new_with_range(-10000.0, 10000.0, 1.0);
+                    g_object_set(G_OBJECT(inputWidget), "hexpand", TRUE, NULL);
                     argument = inputWidget;
                     break;
                 case T_Color:
                     inputWidget = gtk_color_button_new();
+                    g_object_set(G_OBJECT(inputWidget), "hexpand", TRUE, NULL);
                     argument = inputWidget;
                     break;
                 case T_Boolean:
                     inputWidget = gtk_check_button_new();
+                    g_object_set(G_OBJECT(inputWidget), "hexpand", TRUE, NULL);
                     argument = inputWidget;
                     break;
                 case T_Font: {
                     inputWidget = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+                    g_object_set(G_OBJECT(inputWidget), "hexpand", TRUE, NULL);
                     argument = gtk_font_button_new();
                     GtkWidget *button = gtk_button_new_with_label(_("Clear font setting"));
                     gtk_box_pack_start(GTK_BOX(inputWidget), argument, TRUE, TRUE, 0);
@@ -199,18 +205,11 @@ fcitx_config_widget_setup_ui(FcitxConfigWidget *self)
                 case T_Enum: {
                     int i;
                     FcitxConfigEnum *e = &codesc->configEnum;
-#if GTK_CHECK_VERSION(2, 24, 0)
                     inputWidget = gtk_combo_box_text_new();
                     for (i = 0; i < e->enumCount; i ++) {
                         gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(inputWidget), D_(cfdesc->domain, e->enumDesc[i]));
                     }
-#else
-                    inputWidget = gtk_combo_box_new_text();
-                    for (i = 0; i < e->enumCount; i ++)
-                    {
-                        gtk_combo_box_append_text(GTK_COMBO_BOX(inputWidget), D_(cfdesc->domain, e->enumDesc[i]));
-                    }
-#endif
+                    g_object_set(G_OBJECT(inputWidget), "hexpand", TRUE, NULL);
                     argument = inputWidget;
                 }
                 break;
@@ -221,6 +220,8 @@ fcitx_config_widget_setup_ui(FcitxConfigWidget *self)
                     inputWidget = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
                     gtk_box_pack_start(GTK_BOX(inputWidget), button[0], FALSE, TRUE, 0);
                     gtk_box_pack_start(GTK_BOX(inputWidget), button[1], FALSE, TRUE, 0);
+                    g_object_set(G_OBJECT(button[0]), "hexpand", TRUE, NULL);
+                    g_object_set(G_OBJECT(button[1]), "hexpand", TRUE, NULL);
                     argument = g_array_new(FALSE, FALSE, sizeof(void*));
                     g_array_append_val(argument, button[0]);
                     g_array_append_val(argument, button[1]);
@@ -230,6 +231,7 @@ fcitx_config_widget_setup_ui(FcitxConfigWidget *self)
                 case T_Char:
                 case T_String:
                     inputWidget = gtk_entry_new();
+                    g_object_set(G_OBJECT(inputWidget), "hexpand", TRUE, NULL);
                     argument = inputWidget;
                     break;
                 default:
@@ -427,9 +429,9 @@ void sync_filter(FcitxGenericConfig* gconfig, FcitxConfigGroup *group, FcitxConf
 #else
             gtk_color_button_get_rgba(GTK_COLOR_BUTTON(arg), &color);
 #endif
-            r = color.red / 256;
-            g = color.green / 256;
-            b = color.blue / 256;
+            r = color.red * 256;
+            g = color.green * 256;
+            b = color.blue * 256;
             r = RoundColor(r);
             g = RoundColor(g);
             b = RoundColor(b);
@@ -557,7 +559,7 @@ void hash_foreach_cb(gpointer       key,
     int i = context->i;
 
     GtkWidget* label = gtk_label_new(dgettext(widget->parser->domain, subconfig->name));
-    g_object_set(label, "xalign", 0.0f, NULL);
+    g_object_set(G_OBJECT(label),"xalign", 0.0f,  "yalign", 0.0f, "margin", 5, NULL);
 
     GtkWidget *inputWidget = GTK_WIDGET(fcitx_sub_config_widget_new(subconfig));
 
@@ -577,3 +579,39 @@ gboolean fcitx_config_widget_response_cb(GtkDialog *dialog,
     gtk_widget_destroy(GTK_WIDGET(dialog));
     return FALSE;
 }
+
+GtkWidget* fcitx_config_dialog_new(FcitxAddon* addon, GtkWindow* parent)
+{
+    gchar* config_desc_name = g_strdup_printf("%s.desc", addon->name);
+    FcitxConfigFileDesc* cfdesc = get_config_desc(config_desc_name);
+    g_free(config_desc_name);
+    gboolean configurable = (gboolean)(cfdesc != NULL || strlen(addon->subconfig) != 0);
+    if (!configurable) {
+        return NULL;
+    }
+    GtkWidget* dialog = gtk_dialog_new_with_buttons(addon->generalname,
+                                                    parent,
+                                                    GTK_DIALOG_MODAL,
+                                                    GTK_STOCK_CANCEL,
+                                                    GTK_RESPONSE_CANCEL,
+                                                    GTK_STOCK_OK,
+                                                    GTK_RESPONSE_OK,
+                                                    NULL
+                                                );
+
+    gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                         GTK_RESPONSE_OK,
+                                         GTK_RESPONSE_CANCEL,
+                                         -1);
+    gchar* config_file_name = g_strdup_printf("%s.config", addon->name);
+    FcitxConfigWidget* config_widget = fcitx_config_widget_new(cfdesc, "conf", config_file_name, addon->subconfig);
+    GtkWidget* content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+    gtk_box_pack_start(GTK_BOX(content_area), GTK_WIDGET(config_widget), TRUE, TRUE, 0);
+    g_free(config_file_name);
+    gtk_widget_set_size_request(GTK_WIDGET(config_widget), -1, 400);
+
+    g_signal_connect(dialog, "response",
+                    G_CALLBACK(fcitx_config_widget_response_cb),
+                    config_widget);
+    return dialog;
+}
