@@ -37,28 +37,17 @@ enum {
     N_COLUMNS
 };
 
-enum {
-    PAGE_LIST_NAME,
-    PAGE_LIST_PAGE,
-    PAGE_LIST_ICON,
-    PAGE_N_COLUMNS
-};
-
 G_DEFINE_TYPE(FcitxMainWindow, fcitx_main_window, GTK_TYPE_WINDOW)
 
 static void fcitx_main_window_finalize(GObject* object);
 
-static GtkListStore *_fcitx_main_window_create_model();
-
 static void _fcitx_main_window_add_config_file_page(FcitxMainWindow* self);
 
 static void _fcitx_main_window_add_addon_page(FcitxMainWindow* self);
 
 static void _fcitx_main_window_add_im_page(FcitxMainWindow* self);
 
-static void _fcitx_main_window_selection_changed_cb(GtkIconView *iconview, gpointer data);
-
-static ConfigPage* _fcitx_main_window_add_page(FcitxMainWindow* self, const char* name, GtkWidget* widget, const char* stock);
+void _fcitx_main_window_add_page(FcitxMainWindow* self, const char* name, GtkWidget* widget, const char* stock);
 
 static void _fcitx_main_window_addon_selection_changed(GtkTreeSelection *selection, gpointer data);
 
@@ -96,50 +85,17 @@ static void
 fcitx_main_window_init(FcitxMainWindow* self)
 {
     GtkWidget* vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-    GtkWidget* hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-
-    self->pagestore = _fcitx_main_window_create_model();
-    self->pageview = gtk_icon_view_new_with_model(GTK_TREE_MODEL(self->pagestore));
-    
-    gtk_icon_view_set_pixbuf_column(GTK_ICON_VIEW(self->pageview), PAGE_LIST_ICON);
-    gtk_icon_view_set_text_column(GTK_ICON_VIEW(self->pageview), PAGE_LIST_NAME);
-    gtk_icon_view_set_item_orientation(GTK_ICON_VIEW(self->pageview), GTK_ORIENTATION_VERTICAL);
+    self->configNotebook = gtk_notebook_new();
 
     _fcitx_main_window_add_im_page(self);
     _fcitx_main_window_add_config_file_page(self);
     _fcitx_main_window_add_addon_page(self);
 
-    gtk_widget_set_size_request(GTK_WIDGET(self), -1, 500);
+    gtk_widget_set_size_request(GTK_WIDGET(self), -1, 400);
 
-    self->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-    self->pagelabel = gtk_label_new("");
-    gtk_label_set_use_markup(GTK_LABEL(self->pagelabel), true);
-    gtk_misc_set_alignment(GTK_MISC(self->pagelabel), 0, 0.5);
-
-    gtk_box_pack_start(GTK_BOX(self->vbox), self->pagelabel, FALSE, FALSE, 14);
-    GtkWidget* scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-    gtk_container_add(GTK_CONTAINER(scrolledwindow), self->pageview);
-    gtk_box_pack_start(GTK_BOX(hbox), scrolledwindow, FALSE, TRUE, 4);
-    gtk_box_pack_start(GTK_BOX(hbox), self->vbox, TRUE, TRUE, 8);
-    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 8);
+    gtk_box_pack_start(GTK_BOX(vbox), self->configNotebook, TRUE, TRUE, 0);
 
     gtk_container_add(GTK_CONTAINER(self), vbox);
-
-    gtk_icon_view_set_selection_mode(GTK_ICON_VIEW(self->pageview), GTK_SELECTION_SINGLE);
-    gtk_icon_view_set_item_padding(GTK_ICON_VIEW(self->pageview), 0);
-    gtk_icon_view_set_margin(GTK_ICON_VIEW(self->pageview), 0);
-    gtk_icon_view_set_column_spacing(GTK_ICON_VIEW(self->pageview), 0);
-    gtk_icon_view_set_row_spacing(GTK_ICON_VIEW(self->pageview), 0);
-    gtk_icon_view_set_item_width(GTK_ICON_VIEW(self->pageview), 96);
-
-    g_signal_connect(G_OBJECT(self->pageview), "selection-changed",
-                     G_CALLBACK(_fcitx_main_window_selection_changed_cb), self);
-
-    GtkTreePath* path = gtk_tree_model_get_path(GTK_TREE_MODEL(self->pagestore), &self->impage->iter);
-    gtk_icon_view_select_path(GTK_ICON_VIEW(self->pageview), path);
-    gtk_tree_path_free(path);
-
     gtk_window_set_icon_name(GTK_WINDOW(self), "fcitx-configtool");
     gtk_window_set_title(GTK_WINDOW(self), _("Fcitx Config"));
 }
@@ -158,68 +114,20 @@ void fcitx_main_window_finalize(GObject* object)
 {
     FcitxMainWindow* self = FCITX_MAIN_WINDOW(object);
     utarray_free(self->addons);
+
+    G_OBJECT_CLASS (fcitx_main_window_parent_class)->finalize (object);
 }
 
-ConfigPage* _fcitx_main_window_add_page(FcitxMainWindow* self, const char* name, GtkWidget* widget, const char* stock)
+void _fcitx_main_window_add_page(FcitxMainWindow* self, const char* name, GtkWidget* widget, const char* stock)
 {
-    ConfigPage *page = (ConfigPage*) malloc(sizeof(ConfigPage));
-    memset(page, 0, sizeof(ConfigPage));
-
-    page->page = widget;
-
-    g_object_ref(page->page);
-
     gtk_widget_show_all(widget);
 
-    gtk_list_store_append(self->pagestore, &page->iter);
-    gtk_list_store_set(self->pagestore, &page->iter,
-                       0, name,
-                       1, page,
-                       2, gtk_widget_render_icon(self->pageview, stock, GTK_ICON_SIZE_DND, NULL),
-                       -1);
-
-    return page;
+    GtkWidget *plabel = gtk_label_new(name);
+    gtk_notebook_append_page(GTK_NOTEBOOK(self->configNotebook),
+                                widget,
+                                plabel);
 }
 
-void _fcitx_main_window_selection_changed_cb(GtkIconView* iconview, gpointer data)
-{
-    GtkTreeModel *model = gtk_icon_view_get_model(iconview);
-    GtkTreeIter iter;
-    ConfigPage* page;
-    FcitxMainWindow* self = data;
-    
-    GList* list = gtk_icon_view_get_selected_items(iconview);
-
-    if (list) {
-        gchar* title;
-        gtk_tree_model_get_iter(GTK_TREE_MODEL(self->pagestore), &iter, (GtkTreePath*)(list->data));
-        gtk_tree_model_get(model, &iter,
-                           PAGE_LIST_NAME, &title,
-                           PAGE_LIST_PAGE, &page,
-                           -1);
-        
-        gchar* text = g_strdup_printf("<b>%s</b>", title);
-        gtk_label_set_markup(GTK_LABEL(self->pagelabel), text);
-        g_free(text);
-        g_free(title);
-
-        if (self->lastpage)
-            gtk_container_remove(GTK_CONTAINER(self->vbox), self->lastpage->page);
-        gtk_box_pack_end(GTK_BOX(self->vbox), page->page, TRUE, TRUE, 0);
-        gtk_widget_show_all(GTK_WIDGET(self));
-
-        self->lastpage = page;
-    } else {
-        GtkTreePath* path = gtk_tree_model_get_path(GTK_TREE_MODEL(self->pagestore), &self->impage->iter);
-        gtk_icon_view_select_path(GTK_ICON_VIEW(self->pageview), path);
-        gtk_tree_path_free(path);
-    }
-    
-    g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
-    g_list_free (list);
-}
-
-
 void _fcitx_main_window_addon_selection_changed(GtkTreeSelection *selection, gpointer data)
 {
     GtkTreeView *treeView = gtk_tree_selection_get_tree_view(selection);
@@ -245,12 +153,6 @@ void _fcitx_main_window_addon_selection_changed(GtkTreeSelection *selection, gpo
     }
 }
 
-static GtkListStore *_fcitx_main_window_create_model()
-{
-    GtkListStore* store = gtk_list_store_new(PAGE_N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER, GDK_TYPE_PIXBUF);
-    return store;
-}
-
 void _fcitx_main_window_add_config_file_page(FcitxMainWindow* self)
 {
     GtkWidget* vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
@@ -261,81 +163,158 @@ void _fcitx_main_window_add_config_file_page(FcitxMainWindow* self)
                                            "config",
                                            NULL
                                        );
+    g_object_set(G_OBJECT(config_widget), "margin", 5, NULL);
     gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(config_widget), TRUE, TRUE, 0);
 
     GtkWidget* hbuttonbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
     gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, TRUE, 0);
+    g_object_set(G_OBJECT(hbuttonbox), "margin", 5, NULL);
 
     GtkWidget* applybutton = gtk_button_new_from_stock(GTK_STOCK_APPLY);
     gtk_box_pack_start(GTK_BOX(hbuttonbox), applybutton, TRUE, TRUE, 0);
     g_signal_connect(G_OBJECT(applybutton), "clicked", G_CALLBACK(_fcitx_main_window_apply_button_clicked), config_widget);
 
 
-    self->configpage = _fcitx_main_window_add_page(self, _("Global Config"), vbox, GTK_STOCK_PREFERENCES);
+    _fcitx_main_window_add_page(self, _("Global Config"), vbox, GTK_STOCK_PREFERENCES);
 }
 
 void _fcitx_main_window_add_im_page(FcitxMainWindow* self)
 {
     GtkWidget* imwidget = fcitx_im_widget_new();
-    self->impage = _fcitx_main_window_add_page(self, _("Input Method"), imwidget, GTK_STOCK_EDIT);
+    _fcitx_main_window_add_page(self, _("Input Method"), imwidget, GTK_STOCK_EDIT);
+}
+
+gboolean _filter_addon_func(GtkTreeModel *model,
+                            GtkTreeIter  *iter,
+                            gpointer      data)
+{
+    FcitxMainWindow* self = data;
+    FcitxAddon* addon = NULL;
+    const gchar* filter_text = gtk_entry_get_text(GTK_ENTRY(self->filterentry));
+    gtk_tree_model_get(model,
+                       iter,
+                       LIST_ADDON, &addon,
+                       -1);
+
+    gboolean flag = TRUE;
+    if (addon) {
+        if ((addon->category == AC_FRONTEND || addon->advance) && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->advancecheckbox))) {
+            return false;
+        }
+        flag = flag && (strlen(filter_text) == 0
+                 || strstr(addon->name, filter_text)
+                 || strstr(addon->comment, filter_text)
+                 || strstr(addon->generalname, filter_text));
+    }
+    return flag;
+}
+
+void _fcitx_main_window_checkbox_changed(GtkToggleButton* button, gpointer user_data)
+{
+    FcitxMainWindow* self = user_data;
+    gtk_cell_renderer_toggle_set_activatable(GTK_CELL_RENDERER_TOGGLE(self->togglecell), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->advancecheckbox)));
+    gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(self->filtermodel));
+}
+
+static void
+icon_press_cb (GtkEntry       *entry,
+               gint            position,
+               GdkEventButton *event,
+               gpointer        data)
+{
+    gtk_entry_set_text (entry, "");
+}
+
+void _fcitx_main_window_filtertext_changed(GtkEditable* editable, gpointer user_data)
+{
+    FcitxMainWindow* self = user_data;
+    gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(self->filtermodel));
 }
 
 void _fcitx_main_window_add_addon_page(FcitxMainWindow* self)
 {
+    /* load addon */
     FcitxAddon* addon;
     utarray_new(self->addons, &addonicd);
     FcitxAddonsLoad(self->addons);
 
     GtkWidget* vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
-    GtkListStore *store;
-    store = gtk_list_store_new(N_COLUMNS, G_TYPE_POINTER);
+    /* advance check box */
+    self->advancecheckbox = gtk_check_button_new_with_label(_("Advance"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->advancecheckbox), FALSE);
+    g_signal_connect(G_OBJECT(self->advancecheckbox), "toggled", G_CALLBACK(_fcitx_main_window_checkbox_changed), self);
 
-    GtkWidget* swin = gtk_scrolled_window_new(NULL, NULL);
-    gtk_box_pack_start(GTK_BOX(vbox), swin, TRUE, TRUE, 0);
-    g_object_set(swin, "hscrollbar-policy", GTK_POLICY_NEVER, NULL);
-    self->addonview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-    g_object_set(self->addonview, "headers-visible", FALSE, NULL);
-    gtk_container_add(GTK_CONTAINER(swin), self->addonview);
-    GtkCellRenderer *renderer;
-    GtkTreeViewColumn *column;
+    /* filter entry */
+    self->filterentry = gtk_entry_new();
+    gtk_entry_set_icon_from_stock (GTK_ENTRY (self->filterentry),
+                                    GTK_ENTRY_ICON_SECONDARY,
+                                    GTK_STOCK_CLEAR);
+    g_object_set(G_OBJECT(self->filterentry), "margin", 5, NULL);
+#if GTK_CHECK_VERSION(3,2,0)
+    gtk_entry_set_placeholder_text(GTK_ENTRY (self->filterentry), _("Search Addon"));
+#endif
+    g_signal_connect(G_OBJECT(self->filterentry), "icon-press", G_CALLBACK (icon_press_cb), NULL);
+    gtk_box_pack_start(GTK_BOX(vbox), self->filterentry, FALSE, TRUE, 5);
 
-    renderer = gtk_cell_renderer_toggle_new();
-    column = gtk_tree_view_column_new_with_attributes("Enable", renderer, NULL);
-    gtk_tree_view_append_column(GTK_TREE_VIEW(self->addonview), column);
-    gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(column),
-                                       renderer,
+    /* list view */
+    self->addonstore = gtk_list_store_new(N_COLUMNS, G_TYPE_POINTER);
+    for (addon = (FcitxAddon *) utarray_front(self->addons);
+         addon != NULL;
+         addon = (FcitxAddon *) utarray_next(self->addons, addon)) {
+        GtkTreeIter iter;
+        gtk_list_store_append(self->addonstore, &iter);
+        gtk_list_store_set(self->addonstore, &iter, LIST_ADDON, addon, -1);
+    }
+
+    self->filtermodel = gtk_tree_model_filter_new(GTK_TREE_MODEL(self->addonstore), NULL);
+
+    gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(self->filtermodel),
+                                           (GtkTreeModelFilterVisibleFunc) _filter_addon_func,
+                                           self,
+                                           NULL);
+    self->addonview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(self->filtermodel));
+
+    /* add column check box */
+    self->togglecell = gtk_cell_renderer_toggle_new();
+    gtk_cell_renderer_toggle_set_activatable(GTK_CELL_RENDERER_TOGGLE(self->togglecell), FALSE);
+    self->checkboxcolumn = gtk_tree_view_column_new_with_attributes("Enable", self->togglecell, NULL);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(self->addonview), self->checkboxcolumn);
+    gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(self->checkboxcolumn),
+                                       self->togglecell,
                                        _fcitx_main_window_enabled_data_func,
-                                       self->addonview,
+                                       NULL,
                                        NULL);
-    g_signal_connect(G_OBJECT(renderer), "toggled",
-                     G_CALLBACK(_fcitx_main_window_toggled_cb), GTK_TREE_MODEL(store));
+    gtk_cell_renderer_toggle_set_activatable(GTK_CELL_RENDERER_TOGGLE(self->togglecell), FALSE);
 
-    renderer = gtk_cell_renderer_text_new();
-    column = gtk_tree_view_column_new_with_attributes("Name", renderer, NULL);
+    /* add column text */
+    GtkCellRenderer* renderer = gtk_cell_renderer_text_new();
+    GtkTreeViewColumn* column = gtk_tree_view_column_new_with_attributes("Name", renderer, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(self->addonview), column);
     gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(column),
                                        renderer,
                                        _fcitx_main_window_name_data_func,
-                                       self->addonview,
+                                       NULL,
                                        NULL);
 
-    gtk_tree_view_set_model(GTK_TREE_VIEW(self->addonview),
-                            GTK_TREE_MODEL(store));
+    /* add addon list to vbox */
+    GtkWidget* swin = gtk_scrolled_window_new(NULL, NULL);
+    g_object_set(swin, "hscrollbar-policy", GTK_POLICY_NEVER, NULL);
+    g_object_set(self->addonview, "headers-visible", FALSE, NULL);
+    gtk_container_add(GTK_CONTAINER(swin), self->addonview);
+    gtk_box_pack_start(GTK_BOX(vbox), swin, TRUE, TRUE, 0);
+    g_object_set(G_OBJECT(swin), "margin-left", 5, "margin-right", 5, "shadow-type", GTK_SHADOW_IN, NULL);
 
-    g_object_unref(store);
+    g_signal_connect(G_OBJECT(self->togglecell), "toggled",
+                     G_CALLBACK(_fcitx_main_window_toggled_cb), GTK_TREE_MODEL(self->addonstore));
 
-    for (addon = (FcitxAddon *) utarray_front(self->addons);
-            addon != NULL;
-            addon = (FcitxAddon *) utarray_next(self->addons, addon)) {
-        GtkTreeIter iter;
-        store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(self->addonview)));
-        gtk_list_store_append(store, &iter);
-        gtk_list_store_set(store, &iter, LIST_ADDON, addon, -1);
-    }
+    gtk_box_pack_start(GTK_BOX(vbox), self->advancecheckbox, FALSE, TRUE, 0);
+    g_object_set(G_OBJECT(self->advancecheckbox), "margin-left", 5, "margin-right", 5, NULL);
 
+    /* configure button */
     GtkWidget* hbuttonbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
     gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, TRUE, 0);
+    g_object_set(G_OBJECT(hbuttonbox), "margin", 5, NULL);
 
     self->button = gtk_button_new_with_label(_("Configure"));
     gtk_widget_set_sensitive(self->button, FALSE);
@@ -344,11 +323,13 @@ void _fcitx_main_window_add_addon_page(FcitxMainWindow* self)
     g_signal_connect(G_OBJECT(self->button), "clicked", G_CALLBACK(_fcitx_main_window_configure_button_clicked), self);
 
     GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(self->addonview));
-    gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
     g_signal_connect(G_OBJECT(selection), "changed",
                      G_CALLBACK(_fcitx_main_window_addon_selection_changed), self);
+    g_signal_connect(G_OBJECT(self->filterentry), "changed", G_CALLBACK(_fcitx_main_window_filtertext_changed), self);
+    gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(self->filtermodel));
 
-    self->addonpage = _fcitx_main_window_add_page(self, _("Addon"), vbox, GTK_STOCK_ADD);
+    _fcitx_main_window_add_page(self, _("Addon"), vbox, GTK_STOCK_ADD);
 }
 
 static void
@@ -391,15 +372,16 @@ _fcitx_main_window_enabled_data_func(GtkCellLayout   *cell_layout,
                                      GtkTreeIter     *iter,
                                      gpointer         user_data)
 {
-    FcitxAddon* addon;
+    FcitxAddon* addon = NULL;
 
     gtk_tree_model_get(tree_model,
                        iter,
                        LIST_ADDON, &addon,
                        -1);
+
     g_object_set(renderer,
-                 "active", (gboolean) addon->bEnabled,
-                 NULL);
+                "active", addon->bEnabled,
+                NULL);
 }
 
 static void
@@ -442,33 +424,9 @@ void _fcitx_main_window_configure_button_clicked(GtkButton* button, gpointer dat
         gtk_tree_model_get(model, &iter,
                            LIST_ADDON, &addon,
                            -1);
-        gchar* config_desc_name = g_strdup_printf("%s.desc", addon->name);
-        FcitxConfigFileDesc* cfdesc = get_config_desc(config_desc_name);
-        g_free(config_desc_name);
-        gboolean configurable = (gboolean)(cfdesc != NULL || strlen(addon->subconfig) != 0);
-        if (configurable) {
-            GtkWidget* dialog = gtk_dialog_new_with_buttons(addon->generalname,
-                                GTK_WINDOW(self),
-                                GTK_DIALOG_MODAL,
-                                GTK_STOCK_OK,
-                                GTK_RESPONSE_OK,
-                                GTK_STOCK_CANCEL,
-                                GTK_RESPONSE_CANCEL,
-                                NULL
-                                                           );
-
-            gchar* config_file_name = g_strdup_printf("%s.config", addon->name);
-            FcitxConfigWidget* config_widget = fcitx_config_widget_new(cfdesc, "conf", config_file_name, addon->subconfig);
-            GtkWidget* content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-            gtk_box_pack_start(GTK_BOX(content_area), GTK_WIDGET(config_widget), TRUE, TRUE, 0);
-            g_free(config_file_name);
-            gtk_widget_set_size_request(GTK_WIDGET(config_widget), -1, 400);
-
-            g_signal_connect(dialog, "response",
-                             G_CALLBACK(fcitx_config_widget_response_cb),
-                             config_widget);
 
+        GtkWidget* dialog = fcitx_config_dialog_new(addon, GTK_WINDOW(self));
+        if (dialog)
             gtk_widget_show_all(GTK_WIDGET(dialog));
-        }
     }
 }
